unit dropurl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, OLE2, ShellAPI;

type
  // Our Events
  TOnURLDropEvent = procedure(Sender: TObject; URL: String) of object;

  // IDropTarget abstract declaration
  TIDropTarget = class(IUnknown)
  public
    function DragEnter(dataObj: IDataObject; grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult; virtual; stdcall; abstract;
    function DragOver(grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult; virtual; stdcall; abstract;
    function DragLeave: HResult; virtual; stdcall; abstract;
    function Drop(dataObj: IDataObject; grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult; virtual; stdcall; abstract;
  end;

  // IDropTarget Implementation
  TMyDragDrop = class(TIDropTarget)
  private
   refCount 		    : Longint;
   FOnHaveURL               : TOnURLDropEvent;

   NetscapeCommunicatorURL  : integer;
   InternetExplorerURL	    : integer;
   pc 			    : pchar;
   url			    : string[255];

  public
   constructor Create;
   destructor Destroy; override;
   function QueryInterface(const iid: TIID; var obj): HResult; override;
   function AddRef: Longint; override;
   function Release: Longint; override;
   function DragEnter(dataObj: IDataObject; grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult; override;
   function DragOver(grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult; override;
   function DragLeave: HResult; override;
   function Drop(dataObj: IDataObject; grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult; override;
  published
   property OnHaveURL: TOnURLDropEvent read FOnHaveURL write FOnHaveURL;
  end;

  // Our Component
  TDropURL = class(TComponent)
  private
    { Private declarations }
    FApplyTo	: TWinControl;
    FOnDropURL  : TOnURLDropEvent;
    DragDrop    : TMyDragDrop;
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure loaded; override;
    constructor create(AOwner: TComponent); override;
    destructor  destroy; override;
  published
    { Published declarations }
    property TargetControl: TWinControl read FApplyTo write FApplyTo;
    property OnDropURL: TOnURLDropEvent read FOnDropURL write FOnDropURL;
  end;

procedure Register;

implementation

{$R *.RES}

{--- IDragDrop Methods -----------------------------------------------------}

constructor TMyDragDrop.Create;
begin
 inherited Create;

 OleInitialize(NIL);

 refcount := 1;
end;

destructor TMyDragDrop.Destroy;
begin
 // Shut down OLE
 OleUnInitialize;

 inherited Destroy;
end;

function TMyDragDrop.QueryInterface(const iid: TIID; var obj): HResult;
begin
 if IsEqualIID(iid, IID_IDROPTARGET) or IsEqualIID(iid, IID_IUNKNOWN) then
 begin
  Pointer(obj) := self;
  AddRef;
  result := S_OK;
 end
  else
 begin
  Pointer(obj) := NIL;
  result := E_NOINTERFACE;
 end;
end;

function TMyDragDrop.AddRef: Longint;
begin
 inc(refCount);
 result := refCount;
end;

function TMyDragDrop.Release: Longint;
begin
 dec(refCount);
 result := refCount;
 if refCount=0 then Free;
end;

function TMyDragDrop.DragEnter(dataObj: IDataObject; grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult;
var
 res : Integer;
 etc : TFormatETC;
 med : TSTGMedium;

begin
 dataObj.AddRef;

 // Try CF_TEXT format (Netscape, Etc)
 etc.cfFormat 	:= NetscapeCommunicatorURL;
 etc.ptd 	:= NIL;
 etc.dwAspect 	:= DVASPECT_CONTENT;
 etc.lindex 	:= -1;
 etc.tymed 	:= TYMED_HGLOBAL;
 res 		:= dataObj.GetData(etc, med);

 // Try Internet Explorer if that did not work
 if (res <> S_OK) then
 begin
  etc.cfFormat 	:= InternetExplorerURL;
  etc.ptd 	:= NIL;
  etc.dwAspect 	:= DVASPECT_CONTENT;
  etc.lindex 	:= -1;
  etc.tymed 	:= TYMED_HGLOBAL;
  res 		:= dataObj.GetData(etc, med);
 end;

 if (res = S_OK) then
 begin
  // Save the URL data
  pc := GlobalLock(med.hglobal);
  url := strpas(pc);
  GlobalUnlock(med.hGlobal);

  // Pass data back
  dwEffect := DROPEFFECT_COPY;
  result := S_OK;
  exit;
 end;

 result := S_FALSE;
end;

function TMyDragDrop.DragOver(grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult;
begin
 dwEffect := DROPEFFECT_COPY;
 result := S_OK;
end;

function TMyDragDrop.DragLeave: HResult;
begin
 result := S_OK;
end;

function TMyDragDrop.Drop(dataObj: IDataObject; grfKeyState: Longint; pt: TPoint; var dwEffect: Longint): HResult;
begin
 // Simple display code here
 if assigned(FOnHaveURL) then FOnHaveUrl(Self, URL);

 // Set OLE Return Codes
 dwEffect := DROPEFFECT_COPY;
 result := S_OK;
end;


{--- Object Creation -----------------------------------------------------}


constructor TDropURL.Create(AOwner: TComponent);
begin
 // We only need to do custom stuff at run time.
 inherited Create(AOwner);
end;


{--- Object Disposal -----------------------------------------------------}


destructor TDropURL.Destroy;
begin
 // Release DragDrop Object
 if assigned(DragDrop) then DragDrop.Free;

 // Fin
 inherited destroy;
end;


{--- Object Loaded -------------------------------------------------------}


procedure TDropURL.Loaded;
begin
 inherited Loaded;

 // No goodies at design time
 if (csDesigning in ComponentState) then exit;

 // Nothing assigned?
 if not assigned(FApplyTo) then exit;

 // Create OLE DropTarget
 DragDrop := TMyDragDrop.Create;

 // Set the OnHaveURL Event
 DragDrop.OnHaveURL := OnDropURL;

 // Get registered clipboard formats
 DragDrop.NetscapeCommunicatorURL := RegisterClipboardFormat('Netscape Bookmark');
 DragDrop.InternetExplorerURL	  := RegisterClipboardFormat('UniformResourceLocator');

 //res1 := CoLockObjectExternal(DragDrop, true, false);

 // Register our form as a drop target using DragDrop for OLE functions.
 RegisterDragDrop(FApplyTo.Handle, IDropTarget(DragDrop));
end;

{--- Object Registration --------------------------------------------------}

procedure Register;
begin
  RegisterComponents('NetStuff', [TDropURL]);
end;

end.
